@@ -3,11 +3,11 @@ from pymatgen.io.vasp.outputs import BSVasprun
 from pymatgen.io.vasp.inputs import Kpoints, Poscar
 from pymatgen.core.periodic_table import Element
 from functools import reduce
-from collections import OrderedDict
 import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 import time
+from copy import deepcopy
 
 
 class Band:
@@ -48,8 +48,8 @@ class Band:
         self.kpath = kpath
         self.n = n
         self.folder = folder
-        self.spin = spin 
-        self.spin_dict = {'up': Spin.up, 'dowm': Spin.down}
+        self.spin = spin
+        self.spin_dict = {'up': Spin.up, 'down': Spin.down}
         self.bands_dict = self._load_bands()
         self.color_dict = {
             0: '#FF0000',
@@ -359,9 +359,11 @@ class Band:
                         if self.forbitals:
                             element_dict[band][element]['f'] = df[9] + df[10] + \
                                 df[11] + df[12] + df[13] + df[14] + df[15]
-                            element_dict[band][element] = element_dict[band][element].drop(columns=range(16))
+                            element_dict[band][element] = element_dict[band][element].drop(
+                                columns=range(16))
                         else:
-                            element_dict[band][element] = element_dict[band][element].drop(columns=range(9))
+                            element_dict[band][element] = element_dict[band][element].drop(
+                                columns=range(9))
                 else:
                     element_dict[band][element] = df.sum(axis=1).tolist()
 
@@ -400,7 +402,8 @@ class Band:
         for k in kpoints_index:
             ax.axvline(x=k, color='black', alpha=0.7, linewidth=0.5)
 
-        plt.xticks(kpoints_index, kpts_labels)
+        ax.set_xticks(kpoints_index)
+        ax.set_xticklabels(kpts_labels)
 
     def _get_kticks_hse(self, ax, kpath, n):
 
@@ -421,7 +424,7 @@ class Band:
 
         plt.xticks(kpoints_index, kpath)
 
-    def plot_plain(self, ax, color='black', linewidth=1.25):
+    def plot_plain(self, ax, color='black', linewidth=1.25, linestyle='-'):
         """
         This function plots a plain band structure given that the band data
         has already been loaded with the _load_bands() method.
@@ -442,6 +445,7 @@ class Band:
                 band_values,
                 color=color,
                 linewidth=linewidth,
+                linestyle=linestyle,
                 zorder=0,
             )
 
@@ -450,7 +454,7 @@ class Band:
         else:
             self._get_kticks(ax=ax)
 
-        plt.xlim(0, len(wave_vector)-1)
+        ax.set_xlim(0, len(wave_vector)-1)
 
     def plot_spd(self, ax, scale_factor=5, order=['s', 'p', 'd'], color_dict=None, legend=True, linewidth=0.75, band_color='black'):
         """
@@ -521,15 +525,26 @@ class Band:
                     marker='o',
                     markersize=2,
                     linestyle='',
-                    color=color_dict[orbital])
-                )
+                    color=color_dict[orbital]
+                ))
                 legend_labels.append(
                     f'${orbital}$'
                 )
 
+            leg = ax.get_legend()
+
+            if leg is None:
+                handles = legend_lines
+                labels = legend_labels
+            else:
+                handles = [l._legmarker for l in leg.legendHandles]
+                labels = [text._text for text in leg.texts]
+                handles.extend(legend_lines)
+                labels.extend(legend_labels)
+
             ax.legend(
-                legend_lines,
-                legend_labels,
+                handles,
+                labels,
                 ncol=1,
                 loc='upper left',
                 fontsize=5,
@@ -539,7 +554,7 @@ class Band:
                 handletextpad=0.1,
             )
 
-    def plot_atom_orbitals(self, atom_orbital_pairs, ax, scale_factor=5, color_dict=None, legend=True, linewidth=0.75, band_color='black'):
+    def plot_atom_orbitals(self, atom_orbital_pairs, ax, scale_factor=5, color_list=None, legend=True, linewidth=0.75, band_color='black'):
         """
         This function plots the projected band structure of individual orbitals on
         individual atoms given that the band data has already been loaded with the
@@ -566,8 +581,10 @@ class Band:
         projected_dict = self.projected_dict
         wave_vector = range(len(self.bands_dict['band1']))
 
-        if color_dict is None:
+        if color_list is None:
             color_dict = self.color_dict
+        else:
+            color_dict = {i: color for i, color in enumerate(color_list)}
 
         for band in projected_dict:
             for (i, atom_orbital_pair) in enumerate(atom_orbital_pairs):
@@ -656,7 +673,6 @@ class Band:
                 c=color_dict[orbital],
                 s=scale_factor * plot_df[orbital],
                 zorder=1,
-                label=self.orbital_labels[orbital],
             )
 
         if legend:
@@ -675,9 +691,20 @@ class Band:
                     f'{self.orbital_labels[orbital]}'
                 )
 
+            leg = ax.get_legend()
+
+            if leg is None:
+                handles = legend_lines
+                labels = legend_labels
+            else:
+                handles = [l._legmarker for l in leg.legendHandles]
+                labels = [text._text for text in leg.texts]
+                handles.extend(legend_lines)
+                labels.extend(legend_labels)
+
             ax.legend(
-                legend_lines,
-                legend_labels,
+                handles,
+                labels,
                 ncol=1,
                 loc='upper left',
                 fontsize=5,
@@ -687,7 +714,7 @@ class Band:
                 handletextpad=0.1,
             )
 
-    def plot_atoms(self, atoms, ax, scale_factor=5, color_dict=None, legend=True, linewidth=0.75, band_color='black'):
+    def plot_atoms(self, atoms, ax, scale_factor=5, color_list=None, legend=True, linewidth=0.75, band_color='black'):
         """
         This function plots the projected band structure of given orbitals summed
         across all atoms given that the band data has already been loaded with the
@@ -711,8 +738,10 @@ class Band:
 
         atom_dict = self._sum_atoms(atoms=atoms)
 
-        if color_dict is None:
+        if color_list is None:
             color_dict = self.color_dict
+        else:
+            color_dict = {i: color for i, color in enumerate(color_list)}
 
         plot_df = pd.DataFrame(columns=atoms)
         plot_band = []
@@ -723,35 +752,45 @@ class Band:
             plot_band.extend(self.bands_dict[band])
             plot_wave_vec.extend(range(len(atom_dict[band])))
 
-        for atom in atoms:
+        for (i, atom) in enumerate(atoms):
             ax.scatter(
                 plot_wave_vec,
                 plot_band,
-                c=color_dict[atom],
+                c=color_dict[i],
                 s=scale_factor * plot_df[atom],
                 zorder=1,
-                label=atom,
             )
 
         if legend:
             legend_lines = []
             legend_labels = []
-            for atom in atoms:
+            for (i, atom) in enumerate(atoms):
                 legend_lines.append(plt.Line2D(
                     [0],
                     [0],
                     marker='o',
                     markersize=2,
                     linestyle='',
-                    color=color_dict[atom])
+                    color=color_dict[i])
                 )
                 legend_labels.append(
                     f'{atom}'
                 )
 
+            leg = ax.get_legend()
+
+            if leg is None:
+                handles = legend_lines
+                labels = legend_labels
+            else:
+                handles = [l._legmarker for l in leg.legendHandles]
+                labels = [text._text for text in leg.texts]
+                handles.extend(legend_lines)
+                labels.extend(legend_labels)
+
             ax.legend(
-                legend_lines,
-                legend_labels,
+                handles,
+                labels,
                 ncol=1,
                 loc='upper left',
                 fontsize=5,
@@ -761,7 +800,7 @@ class Band:
                 handletextpad=0.1,
             )
 
-    def plot_elements(self, elements, ax, scale_factor=5, color_dict=None, legend=True, linewidth=0.75, band_color='black'):
+    def plot_elements(self, elements, ax, scale_factor=5, color_list=None, legend=True, linewidth=0.75, band_color='black'):
         """
         This function plots the projected band structure on specified element in
         the calculated structure. This is useful for supercells where there are
@@ -786,8 +825,10 @@ class Band:
 
         element_dict = self._sum_elements(elements=elements, orbitals=False)
 
-        if color_dict is None:
+        if color_list is None:
             color_dict = self.color_dict
+        else:
+            color_dict = {i: color for i, color in enumerate(color_list)}
 
         plot_element = {element: [] for element in elements}
         plot_band = []
@@ -804,9 +845,8 @@ class Band:
                 plot_wave_vec,
                 plot_band,
                 c=color_dict[i],
-                s=scale_factor * plot_element[element],
+                s=scale_factor * np.array(plot_element[element]),
                 zorder=1,
-                label=element
             )
 
         if legend:
@@ -825,9 +865,20 @@ class Band:
                     f'{element}'
                 )
 
+            leg = ax.get_legend()
+
+            if leg is None:
+                handles = legend_lines
+                labels = legend_labels
+            else:
+                handles = [l._legmarker for l in leg.legendHandles]
+                labels = [text._text for text in leg.texts]
+                handles.extend(legend_lines)
+                labels.extend(legend_labels)
+
             ax.legend(
-                legend_lines,
-                legend_labels,
+                handles,
+                labels,
                 ncol=1,
                 loc='upper left',
                 fontsize=5,
@@ -837,26 +888,26 @@ class Band:
                 handletextpad=0.1,
             )
 
-    def plot_element_orbitals(self, element_orbital_pairs, ax, scale_factor=5, color_dict=None, legend=True, linewidth=0.75, band_color='black'):
+    def plot_element_orbitals(self, element_orbital_pairs, ax, scale_factor=5, color_list=None, legend=True, linewidth=0.75, band_color='black'):
         """
-        This function plots the projected band structure on chosen orbitals for each 
-        specified element in the calculated structure. This is useful for supercells 
+        this function plots the projected band structure on chosen orbitals for each 
+        specified element in the calculated structure. this is useful for supercells 
         where the are many atoms of the same element and it is inconvienient to manually
-        list each index in the POSCAR.
+        list each index in the poscar.
 
-        Inputs:
+        inputs:
         ----------
-        elements: (list) List of element symbols to project onto
-        orbitals: (list) List of orbitals to plot for each element listed.
-        ax: (matplotlib.pyplot.axis) Axis to plot the data on
-        scale_factor: (float) Factor to scale weights. This changes the size of the
+        elements: (list) list of element symbols to project onto
+        orbitals: (list) list of orbitals to plot for each element listed.
+        ax: (matplotlib.pyplot.axis) axis to plot the data on
+        scale_factor: (float) factor to scale weights. this changes the size of the
             points in the scatter plot
-        color_dict: (dict[str][str]) This option allow the colors of each orbital
-            specified. Should be in the form of:
+        color_dict: (dict[str][str]) this option allow the colors of each orbital
+            specified. should be in the form of:
             {'orbital index': <color>, 'orbital index': <color>, ...}
-        legend: (bool) Determines if the legend should be included or not.
-        linewidth: (float) Line width of the plain band structure plotted in the background
-        band_color: (string) Color of the plain band structure
+        legend: (bool) determines if the legend should be included or not.
+        linewidth: (float) line width of the plain band structure plotted in the background
+        band_color: (string) color of the plain band structure
         """
 
         self.plot_plain(ax=ax, linewidth=linewidth, color=band_color)
@@ -865,10 +916,12 @@ class Band:
 
         element_dict = self._sum_elements(elements=elements, orbitals=True)
 
-        if color_dict is None:
+        if color_list is None:
             color_dict = self.color_dict
+        else:
+            color_dict = {i: color for i, color in enumerate(color_list)}
 
-        plot_element = {element: pd.DataFrame(
+        plot_element = {element: pd.dataframe(
             columns=[range(9)]) for element in elements}
         plot_band = []
         plot_wave_vec = []
@@ -880,7 +933,7 @@ class Band:
                 plot_element[element] = plot_element[element].append(
                     element_dict[band][element])
 
-        for (i, element_orbital_pair) in enumerate(element_orbital_pairs):
+        for i, element_orbital_pair in enumerate(element_orbital_pairs):
             element = element_orbital_pair[0]
             orbital = element_orbital_pair[1]
             ax.scatter(
@@ -889,13 +942,12 @@ class Band:
                 c=color_dict[i],
                 s=scale_factor * plot_element[element][orbital],
                 zorder=1,
-                label=f'{element}({self.orbital_labels[orbital]})'
             )
 
         if legend:
             legend_lines = []
             legend_labels = []
-            for (i, element_orbital_pair) in enumerate(element_orbital_pairs):
+            for i, element_orbital_pair in enumerate(element_orbital_pairs):
                 element = element_orbital_pair[0]
                 orbital = element_orbital_pair[1]
                 legend_lines.append(plt.Line2D(
@@ -910,9 +962,20 @@ class Band:
                     f'{element}({self.orbital_labels[orbital]})'
                 )
 
+            leg = ax.get_legend()
+
+            if leg is None:
+                handles = legend_lines
+                labels = legend_labels
+            else:
+                handles = [l._legmarker for l in leg.legendHandles]
+                labels = [text._text for text in leg.texts]
+                handles.extend(legend_lines)
+                labels.extend(legend_labels)
+
             ax.legend(
-                legend_lines,
-                legend_labels,
+                handles,
+                labels,
                 ncol=1,
                 loc='upper left',
                 fontsize=5,
@@ -979,7 +1042,8 @@ class Band:
 
         for (i, element) in enumerate(elements):
             if self.forbitals:
-                electronic_structure = Element(element).full_electronic_structure
+                electronic_structure = Element(
+                    element).full_electronic_structure
                 if not np.isin('f', electronic_structure):
                     order = order.remove('f')
             for orbital in order:
@@ -1008,9 +1072,20 @@ class Band:
                         f'{element}(${orbital}$)'
                     )
 
+            leg = ax.get_legend()
+
+            if leg is None:
+                handles = legend_lines
+                labels = legend_labels
+            else:
+                handles = [l._legmarker for l in leg.legendHandles]
+                labels = [text._text for text in leg.texts]
+                handles.extend(legend_lines)
+                labels.extend(legend_labels)
+
             ax.legend(
-                legend_lines,
-                legend_labels,
+                handles,
+                labels,
                 ncol=1,
                 loc='upper left',
                 fontsize=5,
@@ -1036,9 +1111,13 @@ def main():
     # bands._get_kticks_hse(ax=ax, kpath='GXWLGK', n=20)
 
     # bands._get_kticks(ax=ax)
-    bands.plot_spd(ax=ax, scale_factor=5)
-    # bands.plot_plain(ax=ax, linewidth=1)
     # bands.plot_atom_orbitals(ax=ax, atom_orbital_pairs=[[0, 0], [1, 3]])
+    # bands.plot_orbitals(ax=ax, orbitals=[1, 2, 3, 4, 5, 6, 7, 8])
+    bands.plot_spd(ax=ax, order=['s'], scale_factor=5)
+    bands.plot_spd(ax=ax, order=['p'], scale_factor=5)
+    bands.plot_spd(ax=ax, order=['d'], scale_factor=5)
+    print(ax.get_legend().legendHandles[1]._color)
+    # bands.plot_plain(ax=ax, linewidth=1)
     # bands.plot_atoms(ax=ax, atoms=[0])
     # element_dict = bands._sum_elements(elements=['In'])
     # bands.plot_element_spd(ax=ax, elements=['Cd'])
@@ -1048,7 +1127,7 @@ def main():
     plt.ylabel('$E - E_F$ $(eV)$', fontsize=6)
     plt.tick_params(labelsize=6, length=1.5)
     plt.tick_params(axis='x', length=0)
-    plt.tight_layout(pad=0.5)
+    # plt.tight_layout(pad=0.5)
     # plt.legend(ncol=3, loc='upper left', fontsize=5)
     # plt.savefig('bs_orbitals.png')
     plt.show()
