@@ -78,10 +78,15 @@ class Dos:
             energies and densities of the system.
         """
 
+        if self.spin == 'up':
+            spin_factor = 1
+        elif self.spin == 'down':
+            spin_factor = -1
+
         tdos = self.vasprun.tdos
         tdos_dict = {
             'energy': np.array(tdos.energies - tdos.efermi),
-            'density': np.array(tdos.densities[self.spin_dict[self.spin]])
+            'density': spin_factor * np.array(tdos.densities[self.spin_dict[self.spin]])
         }
 
         return tdos_dict
@@ -98,13 +103,18 @@ class Dos:
             with the orbital weights for each atom index.
         """
 
+        if self.spin == 'up':
+            spin_factor = 1
+        elif self.spin == 'down':
+            spin_factor = -1
+
         pdos = self.vasprun.pdos
         pdos_dict = {i: [] for i in range(len(pdos))}
         spin = self.spin_dict[self.spin]
 
         for (i, atom) in enumerate(pdos):
             new_dict = {
-                i: atom[orbital][spin] for (i, orbital) in enumerate(atom)
+                i: spin_factor * atom[orbital][spin] for (i, orbital) in enumerate(atom)
             }
 
             if len(list(new_dict.keys())) == 16:
@@ -266,7 +276,54 @@ class Dos:
 
         return spd_df
 
-    def plot_plain(self, ax, linewidth=1.5, fill=True, alpha=0.3, sigma=0.05, energyaxis='y', color='black'):
+    def _set_density_lims(self, ax, tdensity, tenergy, erange, energyaxis, spin):
+        energy_in_plot_index = np.where(
+            (tenergy > erange[0]) & (tenergy < erange[1])
+        )[0]
+        density_in_plot = tdensity[energy_in_plot_index]
+
+        if len(ax.lines) == 0:
+            if energyaxis == 'y':
+                ax.set_ylim(erange)
+                if spin == 'up':
+                    ax.set_xlim(0, np.max(density_in_plot) * 1.1)
+                elif spin == 'down':
+                    ax.set_xlim(np.min(density_in_plot) * 1.1, 0)
+            elif energyaxis == 'x':
+                ax.set_xlim(erange)
+                if spin == 'up':
+                    ax.set_ylim(0, np.max(density_in_plot) * 1.1)
+                elif spin == 'down':
+                    ax.set_ylim(np.min(density_in_plot) * 1.1, 0)
+        elif len(ax.lines) > 0:
+            if energyaxis == 'y':
+                ax.set_ylim(erange)
+                xlims = ax.get_xlim()
+                if xlims[0] == 0:
+                    if spin == 'up':
+                        ax.set_xlim(0, np.max(density_in_plot) * 1.1)
+                    elif spin == 'down':
+                        ax.set_xlim(np.min(density_in_plot) * 1.1, xlims[1])
+                if xlims[1] == 0:
+                    if spin == 'up':
+                        ax.set_xlim(xlims[0], np.max(density_in_plot) * 1.1)
+                    elif spin == 'down':
+                        ax.set_xlim(np.min(density_in_plot) * 1.1, 0)
+            elif energyaxis == 'x':
+                ax.set_xlim(erange)
+                ylims = ax.get_ylim()
+                if ylims[0] == 0:
+                    if spin == 'up':
+                        ax.set_ylim(0, np.max(density_in_plot) * 1.1)
+                    elif spin == 'down':
+                        ax.set_ylim(np.min(density_in_plot) * 1.1, ylims[1])
+                if ylims[1] == 0:
+                    if spin == 'up':
+                        ax.set_ylim(ylims[0], np.max(density_in_plot) * 1.1)
+                    elif spin == 'down':
+                        ax.set_ylim(np.min(density_in_plot) * 1.1, 0)
+
+    def plot_plain(self, ax, linewidth=1.5, fill=True, alpha=0.3, sigma=0.05, energyaxis='y', color='black', erange=[-6, 6]):
         """
         This function plots the total density of states
 
@@ -290,6 +347,15 @@ class Dos:
         else:
             tdensity = tdos_dict['density']
 
+        self._set_density_lims(
+            ax=ax,
+            tdensity=tdensity,
+            tenergy=tdos_dict['energy'],
+            erange=erange,
+            energyaxis=energyaxis,
+            spin=self.spin,
+        )
+
         if energyaxis == 'y':
             ax.plot(
                 tdensity,
@@ -324,7 +390,7 @@ class Dos:
                     alpha=alpha,
                 )
 
-    def plot_spd(self, ax, order=['s', 'p', 'd'], fill=True, alpha=0.3, linewidth=1.5, sigma=0.05, energyaxis='y', color_dict=None, legend=True, total=True):
+    def plot_spd(self, ax, order=['s', 'p', 'd'], fill=True, alpha=0.3, linewidth=1.5, sigma=0.05, energyaxis='y', color_dict=None, legend=True, total=True, erange=[-6, 6]):
         """
         This function plots the total density of states with the projected
         density of states for the total projections of the s, p, and d orbitals.
@@ -366,6 +432,7 @@ class Dos:
                 alpha=alpha,
                 sigma=sigma,
                 energyaxis=energyaxis,
+                erange=erange,
             )
 
         for orbital in order:
@@ -383,7 +450,6 @@ class Dos:
                     tdos_dict['energy'],
                     color=color_dict[orbital],
                     linewidth=linewidth,
-                    label=f'${orbital}$',
                 )
 
                 if fill:
@@ -401,7 +467,6 @@ class Dos:
                     pdensity,
                     color=color_dict[orbital],
                     linewidth=linewidth,
-                    label=f'${orbital}$',
                 )
 
                 if fill:
@@ -429,9 +494,20 @@ class Dos:
                     f'${orbital}$'
                 )
 
+            leg = ax.get_legend()
+
+            if leg is None:
+                handles = legend_lines
+                labels = legend_labels
+            else:
+                handles = [l._legmarker for l in leg.legendHandles]
+                labels = [text._text for text in leg.texts]
+                handles.extend(legend_lines)
+                labels.extend(legend_labels)
+
             ax.legend(
-                legend_lines,
-                legend_labels,
+                handles,
+                labels,
                 ncol=1,
                 loc='upper left',
                 fontsize=5,
@@ -441,7 +517,7 @@ class Dos:
                 handletextpad=0.1,
             )
 
-    def plot_atom_orbitals(self, ax, atom_orbital_pairs, fill=True, alpha=0.3, linewidth=1.5, sigma=0.05, energyaxis='y', color_dict=None, legend=True, total=True):
+    def plot_atom_orbitals(self, ax, atom_orbital_pairs, fill=True, alpha=0.3, linewidth=1.5, sigma=0.05, energyaxis='y', color_list=None, legend=True, total=True, erange=[-6, 6]):
         """
         This function plots the total density of states with the projected
         density of states for the total projections of the s, p, and d orbitals.
@@ -462,8 +538,10 @@ class Dos:
 
         tdos_dict = self.tdos_dict
 
-        if color_dict is None:
+        if color_list is None:
             color_dict = self.color_dict
+        else:
+            color_dict = {i: color for i, color in enumerate(color_list)}
 
         if total:
             self.plot_plain(
@@ -473,6 +551,7 @@ class Dos:
                 alpha=alpha,
                 sigma=sigma,
                 energyaxis=energyaxis,
+                erange=erange,
             )
 
         for i, atom_orbital_pair in enumerate(atom_orbital_pairs):
@@ -493,7 +572,6 @@ class Dos:
                     tdos_dict['energy'],
                     color=color_dict[i],
                     linewidth=linewidth,
-                    label=f'${atom}({self.orbital_labels[orbital]})$',
                 )
 
                 if fill:
@@ -511,7 +589,6 @@ class Dos:
                     pdensity,
                     color=color_dict[i],
                     linewidth=linewidth,
-                    label=f'${atom}({self.orbital_labels[orbital]})$',
                 )
 
                 if fill:
@@ -542,9 +619,20 @@ class Dos:
                     f'{atom}({self.orbital_labels[atom_orbital_pair[1]]})'
                 )
 
+            leg = ax.get_legend()
+
+            if leg is None:
+                handles = legend_lines
+                labels = legend_labels
+            else:
+                handles = [l._legmarker for l in leg.legendHandles]
+                labels = [text._text for text in leg.texts]
+                handles.extend(legend_lines)
+                labels.extend(legend_labels)
+
             ax.legend(
-                legend_lines,
-                legend_labels,
+                handles,
+                labels,
                 ncol=1,
                 loc='upper left',
                 fontsize=5,
@@ -554,7 +642,7 @@ class Dos:
                 handletextpad=0.1,
             )
 
-    def plot_orbitals(self, ax, orbitals, fill=True, alpha=0.3, linewidth=1.5, sigma=0.05, energyaxis='y', color_dict=None, legend=True, total=True):
+    def plot_orbitals(self, ax, orbitals, fill=True, alpha=0.3, linewidth=1.5, sigma=0.05, energyaxis='y', color_dict=None, legend=True, total=True, erange=[-6, 6]):
         """
         This function plots the total density of states with the projected
         density of states for the total projections of the s, p, and d orbitals.
@@ -587,6 +675,7 @@ class Dos:
                 alpha=alpha,
                 sigma=sigma,
                 energyaxis=energyaxis,
+                erange=erange,
             )
 
         for i, orbital in enumerate(orbitals):
@@ -648,9 +737,20 @@ class Dos:
                     f'{self.orbital_labels[orbital]}'
                 )
 
+            leg = ax.get_legend()
+
+            if leg is None:
+                handles = legend_lines
+                labels = legend_labels
+            else:
+                handles = [l._legmarker for l in leg.legendHandles]
+                labels = [text._text for text in leg.texts]
+                handles.extend(legend_lines)
+                labels.extend(legend_labels)
+
             ax.legend(
-                legend_lines,
-                legend_labels,
+                handles,
+                labels,
                 ncol=1,
                 loc='upper left',
                 fontsize=5,
@@ -660,7 +760,7 @@ class Dos:
                 handletextpad=0.1,
             )
 
-    def plot_atoms(self, ax, atoms, fill=True, alpha=0.3, linewidth=1.5, sigma=0.05, energyaxis='y', color_dict=None, legend=True, total=True):
+    def plot_atoms(self, ax, atoms, fill=True, alpha=0.3, linewidth=1.5, sigma=0.05, energyaxis='y', color_list=None, legend=True, total=True, erange=[-6, 6]):
         """
         This function plots the total density of states with the projected
         density of states on the given atoms.
@@ -683,8 +783,10 @@ class Dos:
         atom_df = self._sum_atoms()
         tdos_dict = self.tdos_dict
 
-        if color_dict is None:
+        if color_list is None:
             color_dict = self.color_dict
+        else:
+            color_dict = {i: color for i, color in enumerate(color_list)}
 
         if total:
             self.plot_plain(
@@ -694,6 +796,7 @@ class Dos:
                 alpha=alpha,
                 sigma=sigma,
                 energyaxis=energyaxis,
+                erange=erange,
             )
 
         for i, atom in enumerate(atoms):
@@ -711,7 +814,6 @@ class Dos:
                     tdos_dict['energy'],
                     color=color_dict[i],
                     linewidth=linewidth,
-                    label=atom,
                 )
 
                 if fill:
@@ -729,7 +831,6 @@ class Dos:
                     pdensity,
                     color=color_dict[i],
                     linewidth=linewidth,
-                    label=atom,
                 )
 
                 if fill:
@@ -757,9 +858,20 @@ class Dos:
                     f'{atom}'
                 )
 
+            leg = ax.get_legend()
+
+            if leg is None:
+                handles = legend_lines
+                labels = legend_labels
+            else:
+                handles = [l._legmarker for l in leg.legendHandles]
+                labels = [text._text for text in leg.texts]
+                handles.extend(legend_lines)
+                labels.extend(legend_labels)
+
             ax.legend(
-                legend_lines,
-                legend_labels,
+                handles,
+                labels,
                 ncol=1,
                 loc='upper left',
                 fontsize=5,
@@ -769,7 +881,7 @@ class Dos:
                 handletextpad=0.1,
             )
 
-    def plot_elements(self, ax, elements, fill=True, alpha=0.3, linewidth=1.5, sigma=0.05, energyaxis='y', color_dict=None, legend=True, total=True):
+    def plot_elements(self, ax, elements, fill=True, alpha=0.3, linewidth=1.5, sigma=0.05, energyaxis='y', color_list=None, legend=True, total=True, erange=[-6, 6]):
         """
         This function plots the total density of states with the projected
         density of states for the projection onto specified elements. This is 
@@ -794,8 +906,10 @@ class Dos:
             elements=elements, orbitals=False, spd=False)
         tdos_dict = self.tdos_dict
 
-        if color_dict is None:
+        if color_list is None:
             color_dict = self.color_dict
+        else:
+            color_dict = {i: color for i, color in enumerate(color_list)}
 
         if total:
             self.plot_plain(
@@ -805,6 +919,7 @@ class Dos:
                 alpha=alpha,
                 sigma=sigma,
                 energyaxis=energyaxis,
+                erange=erange,
             )
 
         for i, element in enumerate(elements):
@@ -822,7 +937,6 @@ class Dos:
                     tdos_dict['energy'],
                     color=color_dict[i],
                     linewidth=linewidth,
-                    label=f'{element}',
                 )
 
                 if fill:
@@ -840,7 +954,6 @@ class Dos:
                     pdensity,
                     color=color_dict[i],
                     linewidth=linewidth,
-                    label=f'{element}',
                 )
 
                 if fill:
@@ -868,9 +981,20 @@ class Dos:
                     f'{element}'
                 )
 
+            leg = ax.get_legend()
+
+            if leg is None:
+                handles = legend_lines
+                labels = legend_labels
+            else:
+                handles = [l._legmarker for l in leg.legendHandles]
+                labels = [text._text for text in leg.texts]
+                handles.extend(legend_lines)
+                labels.extend(legend_labels)
+
             ax.legend(
-                legend_lines,
-                legend_labels,
+                handles,
+                labels,
                 ncol=1,
                 loc='upper left',
                 fontsize=5,
@@ -880,7 +1004,7 @@ class Dos:
                 handletextpad=0.1,
             )
 
-    def plot_element_orbitals(self, ax, element_orbital_pairs, fill=True, alpha=0.3, linewidth=1.5, sigma=0.05, energyaxis='y', color_dict=None, legend=True, total=True):
+    def plot_element_orbitals(self, ax, element_orbital_pairs, fill=True, alpha=0.3, linewidth=1.5, sigma=0.05, energyaxis='y', color_list=None, legend=True, total=True, erange=[-6, 6]):
         """
         This function plots the total density of states with the projected
         density of states onto the chosen orbitals of specified elements. This is 
@@ -908,8 +1032,10 @@ class Dos:
             elements=elements, orbitals=True, spd=False)
         tdos_dict = self.tdos_dict
 
-        if color_dict is None:
+        if color_list is None:
             color_dict = self.color_dict
+        else:
+            color_dict = {i: color for i, color in enumerate(color_list)}
 
         if total:
             self.plot_plain(
@@ -919,6 +1045,7 @@ class Dos:
                 alpha=alpha,
                 sigma=sigma,
                 energyaxis=energyaxis,
+                erange=erange,
             )
 
         for (i, element_orbital_pair) in enumerate(element_orbital_pairs):
@@ -938,7 +1065,6 @@ class Dos:
                     tdos_dict['energy'],
                     color=color_dict[i],
                     linewidth=linewidth,
-                    label=f'{element}({self.orbital_labels[orbital]})',
                 )
 
                 if fill:
@@ -956,7 +1082,6 @@ class Dos:
                     pdensity,
                     color=color_dict[i],
                     linewidth=linewidth,
-                    label=f'{element}({self.orbital_labels[orbital]})',
                 )
 
                 if fill:
@@ -986,9 +1111,20 @@ class Dos:
                     f'{element}({self.orbital_labels[orbital]})'
                 )
 
+            leg = ax.get_legend()
+
+            if leg is None:
+                handles = legend_lines
+                labels = legend_labels
+            else:
+                handles = [l._legmarker for l in leg.legendHandles]
+                labels = [text._text for text in leg.texts]
+                handles.extend(legend_lines)
+                labels.extend(legend_labels)
+
             ax.legend(
-                legend_lines,
-                legend_labels,
+                handles,
+                labels,
                 ncol=1,
                 loc='upper left',
                 fontsize=5,
@@ -998,7 +1134,7 @@ class Dos:
                 handletextpad=0.1,
             )
 
-    def plot_element_spd(self, ax, elements, order=['s', 'p', 'd'], fill=True, alpha=0.3, linewidth=1.5, sigma=0.05, energyaxis='y', color_dict=None, legend=True, total=True):
+    def plot_element_spd(self, ax, elements, order=['s', 'p', 'd'], fill=True, alpha=0.3, linewidth=1.5, sigma=0.05, energyaxis='y', color_dict=None, legend=True, total=True, erange=[-6, 6]):
         """
         This function plots the total density of states with the projected
         density of states onto the s, p, and d orbitals of specified elements. 
@@ -1044,6 +1180,7 @@ class Dos:
                 alpha=alpha,
                 sigma=sigma,
                 energyaxis=energyaxis,
+                erange=erange,
             )
 
         for element in elements:
@@ -1062,7 +1199,6 @@ class Dos:
                         tdos_dict['energy'],
                         color=color_dict[orbital],
                         linewidth=linewidth,
-                        label=f'{element}({orbital})',
                     )
 
                     if fill:
@@ -1080,7 +1216,6 @@ class Dos:
                         pdensity,
                         color=color_dict[orbital],
                         linewidth=linewidth,
-                        label=f'{element}({orbital})',
                     )
 
                     if fill:
@@ -1109,9 +1244,20 @@ class Dos:
                         f'{element}(${orbital}$)'
                     )
 
+            leg = ax.get_legend()
+
+            if leg is None:
+                handles = legend_lines
+                labels = legend_labels
+            else:
+                handles = [l._legmarker for l in leg.legendHandles]
+                labels = [text._text for text in leg.texts]
+                handles.extend(legend_lines)
+                labels.extend(legend_labels)
+
             ax.legend(
-                legend_lines,
-                legend_labels,
+                handles,
+                labels,
                 ncol=1,
                 loc='upper left',
                 fontsize=5,
@@ -1178,10 +1324,13 @@ class Dos:
 def main():
     fig = plt.figure(figsize=(4, 8), dpi=200)
     ax = fig.add_subplot(111)
-    plt.ylim(-6, 6)
+    # plt.xlim(-6, 6)
     ax.margins(x=0.005, y=0.005)
     dos = Dos(folder='../../vaspvis_data/dos')
-    dos.plot_spd(ax=ax, energyaxis='y')
+    dos.plot_spd(ax=ax, energyaxis='x')
+    dos.tdos_dict['density'] = -1 * dos.tdos_dict['density']
+    dos.spin = 'down'
+    dos.plot_spd(ax=ax, energyaxis='x', erange=[-6, 9], legend=False)
     # dos.plot_layers(ax=ax)
     # dos.plot_atoms(ax=ax, atoms=[0, 1], sigma=0.1, fill=True, energyaxis='y')
     plt.show()
